@@ -86,7 +86,7 @@ get_flatpak_subpaths_from_deploy_subpaths (const char * const *subpaths)
     g_ptr_array_add (resolved_subpaths, g_build_filename ("/files", subpaths[i], NULL));
   g_ptr_array_add (resolved_subpaths, NULL);
 
-  return (char **)g_ptr_array_free (g_steal_pointer (&resolved_subpaths), FALSE);
+  return (char **) g_ptr_array_free (g_steal_pointer (&resolved_subpaths), FALSE);
 }
 
 /* Add related refs specified in the metadata of @ref to @all_refs, also
@@ -168,7 +168,7 @@ add_related (GHashTable   *all_refs,
       ext_commit = flatpak_deploy_data_get_commit (ext_deploy_data);
       ext_subpaths = flatpak_deploy_data_get_subpaths (ext_deploy_data);
       resolved_ext_subpaths = get_flatpak_subpaths_from_deploy_subpaths (ext_subpaths);
-      c_s = commit_and_subpaths_new (ext_commit, (const char * const *)resolved_ext_subpaths);
+      c_s = commit_and_subpaths_new (ext_commit, (const char * const *) resolved_ext_subpaths);
 
       g_hash_table_insert (all_collection_ids, g_strdup (ext_collection_id), g_strdup (ext_remote));
       ext_collection_ref = ostree_collection_ref_new (ext_collection_id, ext->ref);
@@ -238,7 +238,7 @@ add_runtime (GHashTable   *all_refs,
   runtime_commit = flatpak_deploy_data_get_commit (runtime_deploy_data);
   runtime_subpaths = flatpak_deploy_data_get_subpaths (runtime_deploy_data);
   resolved_runtime_subpaths = get_flatpak_subpaths_from_deploy_subpaths (runtime_subpaths);
-  c_s = commit_and_subpaths_new (runtime_commit, (const char * const *)resolved_runtime_subpaths);
+  c_s = commit_and_subpaths_new (runtime_commit, (const char * const *) resolved_runtime_subpaths);
 
   g_hash_table_insert (all_collection_ids, g_strdup (runtime_collection_id), g_strdup (runtime_remote));
   runtime_collection_ref = ostree_collection_ref_new (runtime_collection_id, runtime_ref);
@@ -280,7 +280,6 @@ ostree_create_usb (GOptionContext *context,
   OstreeRepoMode mode = OSTREE_REPO_MODE_ARCHIVE;
 
   g_debug ("%s: Creating repository in mode %u", G_STRFUNC, mode);
-
   g_autoptr(OstreeRepo) dest_repo = ostree_repo_create_at (mount_root_dfd, dest_repo_path,
                                                            mode, NULL, cancellable, error);
 
@@ -307,7 +306,6 @@ ostree_create_usb (GOptionContext *context,
   GLNX_HASH_TABLE_FOREACH_KV (all_refs, OstreeCollectionRef *, c_r, CommitAndSubpaths *, c_s)
   {
     GVariantBuilder builder;
-
     g_autoptr(GVariant) opts = NULL;
     OstreeRepoPullFlags flags = OSTREE_REPO_PULL_FLAGS_MIRROR;
     GVariantBuilder refs_builder;
@@ -647,7 +645,6 @@ flatpak_builtin_create_usb (int argc, char **argv, GCancellable *cancellable, GE
         return FALSE;
       if (!add_related (all_refs, all_collection_ids, installed_ref, dir, cancellable, error))
         return FALSE;
-
     }
 
   g_assert (dir);
@@ -741,6 +738,11 @@ flatpak_builtin_create_usb (int argc, char **argv, GCancellable *cancellable, GE
       }
   }
 
+  /* Delete the local source repo summary if it exists. Old versions of this
+   * command erroneously created it and if it's outdated that causes problems. */
+  if (!flatpak_dir_update_summary (dir, TRUE, cancellable, error))
+    return FALSE;
+
   /* Now use code copied from `ostree create-usb` to do the actual copying. We
    * can't just call out to `ostree` because (a) flatpak doesn't have a
    * dependency on the ostree command line tools and (b) we need to only pull
