@@ -54,12 +54,14 @@ static GOptionEntry options[] = {
 };
 
 static Column all_columns[] = {
-  { "description",  N_("Description"),    N_("Show the description"),    1, FLATPAK_ELLIPSIZE_MODE_END, 1, 1 },
-  { "application",  N_("Application"),    N_("Show the application ID"), 1, FLATPAK_ELLIPSIZE_MODE_START, 0, 1 },
+  { "name",         N_("Name"),           N_("Show the name"),           1, FLATPAK_ELLIPSIZE_MODE_END, 1, 1 },
+  { "description",  N_("Description"),    N_("Show the description"),    1, FLATPAK_ELLIPSIZE_MODE_END, 1, 0 },
+  { "application",  N_("Application ID"), N_("Show the application ID"), 1, FLATPAK_ELLIPSIZE_MODE_START, 0, 1 },
   { "version",      N_("Version"),        N_("Show the version"),        1, FLATPAK_ELLIPSIZE_MODE_NONE, 1, 1 },
   { "branch",       N_("Branch"),         N_("Show the branch"),         1, FLATPAK_ELLIPSIZE_MODE_NONE, 0, 1 },
-  { "arch",         N_("Arch"),           N_("Show the architecture"),   1, FLATPAK_ELLIPSIZE_MODE_NONE, 0, 1 },
-  { "origin",       N_("Origin"),         N_("Show the origin remote"),  1, FLATPAK_ELLIPSIZE_MODE_NONE, 1, 1 },
+  { "arch",         N_("Arch"),           N_("Show the architecture"),   1, FLATPAK_ELLIPSIZE_MODE_NONE, 0, 1, 1 },
+  { "runtime",      N_("Runtime"),        N_("Show the used runtime"),   1, FLATPAK_ELLIPSIZE_MODE_START, 0, 0 },
+  { "origin",       N_("Origin"),         N_("Show the origin remote"),  1, FLATPAK_ELLIPSIZE_MODE_NONE, 1, 1, 1 },
   { "installation", N_("Installation"),   N_("Show the installation"),   1, FLATPAK_ELLIPSIZE_MODE_NONE, 1, 0 },
   { "ref",          N_("Ref"),            N_("Show the ref"),            1, FLATPAK_ELLIPSIZE_MODE_NONE, 1, 0 },
   { "active",       N_("Active commit"),  N_("Show the active commit"),  1, FLATPAK_ELLIPSIZE_MODE_NONE, 1, 0 },
@@ -124,11 +126,11 @@ join_strv (char **a, char **b)
 }
 
 static gboolean
-find_refs_for_dir (FlatpakDir *dir,
-                   GStrv *apps,
-                   GStrv *runtimes,
+find_refs_for_dir (FlatpakDir   *dir,
+                   GStrv        *apps,
+                   GStrv        *runtimes,
                    GCancellable *cancellable,
-                   GError **error)
+                   GError      **error)
 {
   if (flatpak_dir_ensure_repo (dir, cancellable, NULL))
     {
@@ -142,13 +144,13 @@ find_refs_for_dir (FlatpakDir *dir,
 }
 
 static gboolean
-print_table_for_refs (gboolean print_apps,
-                      GPtrArray * refs_array,
-                      const char *arch,
-                      const char *app_runtime,
-                      Column *columns,
+print_table_for_refs (gboolean      print_apps,
+                      GPtrArray   * refs_array,
+                      const char   *arch,
+                      const char   *app_runtime,
+                      Column       *columns,
                       GCancellable *cancellable,
-                      GError **error)
+                      GError      **error)
 {
   FlatpakTablePrinter *printer;
   int i;
@@ -163,7 +165,8 @@ print_table_for_refs (gboolean print_apps,
 
   printer = flatpak_table_printer_new ();
 
-  flatpak_table_printer_set_columns (printer, columns);
+  flatpak_table_printer_set_columns (printer, columns,
+                                     opt_cols == NULL && !opt_show_details);
 
   if (app_runtime)
     {
@@ -205,6 +208,7 @@ print_table_for_refs (gboolean print_apps,
           const char *appdata_name;
           const char *appdata_summary;
           const char *appdata_version;
+          const char *runtime;
           g_autofree char *latest = NULL;
           g_autofree const char **subpaths = NULL;
           int k;
@@ -222,9 +226,10 @@ print_table_for_refs (gboolean print_apps,
           if (deploy_data == NULL)
             continue;
 
+         runtime = flatpak_deploy_data_get_runtime (deploy_data);
+
           if (app_runtime)
             {
-              const char *runtime = flatpak_deploy_data_get_runtime (deploy_data);
               if (runtime)
                 {
                   g_auto(GStrv) pref = g_strsplit (runtime, "/", 3);
@@ -280,21 +285,23 @@ print_table_for_refs (gboolean print_apps,
 
           for (k = 0; columns[k].name; k++)
             {
-              if (strcmp (columns[k].name, "description") == 0)
+              if (strcmp (columns[k].name, "name") == 0)
                 {
-                  g_autofree char *description = NULL;
                   const char *name = appdata_name ? appdata_name : strrchr (parts[1], '.') + 1;
 
-                  if (appdata_summary)
-                    description =  g_strconcat (name, " - ", appdata_summary, NULL);
-                  else
-                    description =  g_strdup (name);
+                  flatpak_table_printer_add_column (printer, name);
+                }
+              else if (strcmp (columns[k].name, "description") == 0)
+                {
+                  const char *description = appdata_summary ? appdata_summary : "";
                   flatpak_table_printer_add_column (printer, description);
                 }
               else if (strcmp (columns[k].name, "version") == 0)
                 flatpak_table_printer_add_column (printer, appdata_version ? appdata_version : "");
               else if (strcmp (columns[k].name, "installation") == 0)
                 flatpak_table_printer_add_column (printer, flatpak_dir_get_name_cached (dir));
+              else if (strcmp (columns[k].name, "runtime") == 0)
+                flatpak_table_printer_add_column (printer, runtime ? runtime : "");
               else if (strcmp (columns[k].name, "ref") == 0)
                 flatpak_table_printer_add_column (printer, partial_ref);
               else if (strcmp (columns[k].name, "application") == 0)
@@ -346,7 +353,7 @@ print_table_for_refs (gboolean print_apps,
                   subpaths = flatpak_deploy_data_get_subpaths (deploy_data);
                   if (subpaths[0] != NULL)
                     {
-                      g_autofree char *paths = g_strjoinv (" ", (char **)subpaths);
+                      g_autofree char *paths = g_strjoinv (" ", (char **) subpaths);
                       g_autofree char *value = g_strconcat ("partial (", paths, ")", NULL);
                       flatpak_table_printer_append_with_comma (printer, value);
                     }
@@ -363,7 +370,7 @@ print_table_for_refs (gboolean print_apps,
         }
     }
 
-  flatpak_table_printer_sort (printer, (GCompareFunc)flatpak_compare_ref);
+  flatpak_table_printer_sort (printer, (GCompareFunc) flatpak_compare_ref);
 
   if (flatpak_table_printer_get_current_row (printer) > 0)
     {
@@ -378,14 +385,14 @@ print_table_for_refs (gboolean print_apps,
 }
 
 static gboolean
-print_installed_refs (gboolean app,
-                      gboolean runtime,
-                      GPtrArray *dirs,
-                      const char *arch,
-                      const char *app_runtime,
-                      Column *cols,
+print_installed_refs (gboolean      app,
+                      gboolean      runtime,
+                      GPtrArray    *dirs,
+                      const char   *arch,
+                      const char   *app_runtime,
+                      Column       *cols,
                       GCancellable *cancellable,
-                      GError **error)
+                      GError      **error)
 {
   g_autoptr(GPtrArray) refs_array = NULL;
   int i;
