@@ -42,8 +42,14 @@ static gboolean opt_no_deps;
 static gboolean opt_enable;
 static gboolean opt_update_metadata;
 static gboolean opt_disable;
+static gboolean opt_no_filter;
 static int opt_prio = -1;
+static char *opt_filter;
 static char *opt_title;
+static char *opt_comment;
+static char *opt_description;
+static char *opt_homepage;
+static char *opt_icon;
 static char *opt_default_branch;
 static char *opt_url;
 static char *opt_collection_id = NULL;
@@ -66,9 +72,15 @@ static GOptionEntry common_options[] = {
   { "no-use-for-deps", 0, 0, G_OPTION_ARG_NONE, &opt_no_deps, N_("Mark the remote as don't use for deps"), NULL },
   { "prio", 0, 0, G_OPTION_ARG_INT, &opt_prio, N_("Set priority (default 1, higher is more prioritized)"), N_("PRIORITY") },
   { "title", 0, 0, G_OPTION_ARG_STRING, &opt_title, N_("A nice name to use for this remote"), N_("TITLE") },
+  { "comment", 0, 0, G_OPTION_ARG_STRING, &opt_comment, N_("A one-line comment for this remote"), N_("COMMENT") },
+  { "description", 0, 0, G_OPTION_ARG_STRING, &opt_description, N_("A full-paragraph description for this remote"), N_("DESCRIPTION") },
+  { "homepage", 0, 0, G_OPTION_ARG_STRING, &opt_homepage, N_("URL for a website for this remote"), N_("URL") },
+  { "icon", 0, 0, G_OPTION_ARG_STRING, &opt_icon, N_("URL for an icon for this remote"), N_("URL") },
   { "default-branch", 0, 0, G_OPTION_ARG_STRING, &opt_default_branch, N_("Default branch to use for this remote"), N_("BRANCH") },
   { "collection-id", 0, 0, G_OPTION_ARG_STRING, &opt_collection_id, N_("Collection ID"), N_("COLLECTION-ID") },
   { "gpg-import", 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &opt_gpg_import, N_("Import GPG key from FILE (- for stdin)"), N_("FILE") },
+  { "no-filter", 0, 0, G_OPTION_ARG_NONE, &opt_no_filter, N_("Disable local filter"), NULL },
+  { "filter", 0, 0, G_OPTION_ARG_FILENAME, &opt_filter, N_("Set path to local filter FILE"), N_("FILE") },
   { "disable", 0, 0, G_OPTION_ARG_NONE, &opt_disable, N_("Disable the remote"), NULL },
   { NULL }
 };
@@ -124,6 +136,34 @@ get_config_from_opts (FlatpakDir *dir, const char *remote_name, gboolean *change
       *changed = TRUE;
     }
 
+  if (opt_comment)
+    {
+      g_key_file_set_string (config, group, "xa.comment", opt_comment);
+      g_key_file_set_boolean (config, group, "xa.comment-is-set", TRUE);
+      *changed = TRUE;
+    }
+
+  if (opt_description)
+    {
+      g_key_file_set_string (config, group, "xa.description", opt_description);
+      g_key_file_set_boolean (config, group, "xa.description-is-set", TRUE);
+      *changed = TRUE;
+    }
+
+  if (opt_homepage)
+    {
+      g_key_file_set_string (config, group, "xa.homepage", opt_homepage);
+      g_key_file_set_boolean (config, group, "xa.homepage-is-set", TRUE);
+      *changed = TRUE;
+    }
+
+  if (opt_icon)
+    {
+      g_key_file_set_string (config, group, "xa.icon", opt_icon);
+      g_key_file_set_boolean (config, group, "xa.icon-is-set", TRUE);
+      *changed = TRUE;
+    }
+
   if (opt_default_branch)
     {
       g_key_file_set_string (config, group, "xa.default-branch", opt_default_branch);
@@ -131,6 +171,12 @@ get_config_from_opts (FlatpakDir *dir, const char *remote_name, gboolean *change
       *changed = TRUE;
     }
 
+  if (opt_filter || opt_no_filter)
+    {
+      g_key_file_set_string (config, group, "xa.filter", opt_no_filter ? "" : opt_filter);
+      *changed = TRUE;
+    }
+
   if (opt_no_enumerate)
     {
       g_key_file_set_boolean (config, group, "xa.noenumerate", TRUE);
