@@ -185,21 +185,77 @@ flatpak_builtin_run (int argc, char **argv, GCancellable *cancellable, GError **
     {
       g_autoptr(FlatpakDeploy) runtime_deploy = NULL;
       g_autoptr(GError) local_error2 = NULL;
+      g_autoptr(GPtrArray) ref_dir_pairs = NULL;
+      RefDirPair *chosen_pair;
 
-      runtime_ref = flatpak_compose_ref (FALSE, id, branch, arch, error);
-      if (runtime_ref == NULL)
-        return FALSE;
+      /* Whereas for apps we want to default to using the "current" one (see
+       * flatpak-make-current(1)) runtimes don't have a concept of currentness.
+       * So prompt if there's ambiguity about which branch to use */
+      ref_dir_pairs = g_ptr_array_new_with_free_func ((GDestroyNotify) ref_dir_pair_free);
+      for (i = 0; i < dirs->len; i++)
+        {
+          FlatpakDir *dir = g_ptr_array_index (dirs, i);
+          g_auto(GStrv) refs = NULL;
+          char **iter;
+
+          refs = flatpak_dir_find_installed_refs (dir, id, branch, arch, FLATPAK_KINDS_RUNTIME,
+                                                  FIND_MATCHING_REFS_FLAGS_NONE, error);
+          if (refs == NULL)
+            return FALSE;
+          else if (g_strv_length (refs) == 0)
+            continue;
+
+          for (iter = refs; iter && *iter; iter++)
+            {
+              const char *ref = *iter;
+              RefDirPair *pair;
+
+              pair = ref_dir_pair_new (ref, dir);
+              g_ptr_array_add (ref_dir_pairs, pair);
+            }
+        }
+
+      if (ref_dir_pairs->len > 0)
+        {
+          g_autoptr(GPtrArray) chosen_pairs = NULL;
+          g_autoptr(GPtrArray) chosen_dir_array = NULL;
+
+          chosen_pairs = g_ptr_array_new ();
+
+          if (!flatpak_resolve_matching_installed_refs (TRUE, TRUE, ref_dir_pairs, id, chosen_pairs, error))
+            return FALSE;
+
+          g_assert (chosen_pairs->len == 1);
+          chosen_pair = g_ptr_array_index (chosen_pairs, 0);
+
+          /* For runtimes we don't need to pass a FlatpakDeploy object to
+           * flatpak_run_app(), but get it anyway because we don't want to run
+           * something that's not deployed */
+          chosen_dir_array = g_ptr_array_new ();
+          g_ptr_array_add (chosen_dir_array, chosen_pair->dir);
+          runtime_deploy = flatpak_find_deploy_for_ref_in (chosen_dir_array, chosen_pair->ref,
+                                                           opt_commit ? opt_commit : opt_runtime_commit,
+                                                           cancellable, &local_error2);
+        }
 
-      runtime_deploy = flatpak_find_deploy_for_ref_in (dirs, runtime_ref, opt_commit ? opt_commit : opt_runtime_commit, cancellable, &local_error2);
       if (runtime_deploy == NULL)
         {
           /* Report old app-kind error, as its more likely right */
           if (local_error != NULL)
             g_propagate_error (error, g_steal_pointer (&local_error));
-          else
+          else if (local_error2 != NULL)
             g_propagate_error (error, g_steal_pointer (&local_error2));
+          else
+            flatpak_fail_error (error, FLATPAK_ERROR_NOT_INSTALLED,
+                                _("runtime/%s/%s/%s not installed"),
+                                id ?: "*unspecified*",
+                                arch ?: "*unspecified*",
+                                branch ?: "*unspecified*");
           return FALSE;
         }
+
+      runtime_ref = g_strdup (chosen_pair->ref);
+
       /* Clear app-kind error */
       g_clear_error (&local_error);
     }
@@ -302,7 +358,6 @@ flatpak_complete_run (FlatpakCompletion *completion)
         }
 
       break;
-
     }
 
   return TRUE;
