@@ -50,7 +50,7 @@ free_cell (gpointer data)
 typedef struct
 {
   GPtrArray *cells;
-  char *key;
+  char      *key;
 } Row;
 
 static void
@@ -65,9 +65,11 @@ free_row (gpointer data)
 
 typedef struct
 {
-  char *title;
-  gboolean expand;
+  char                *title;
+  gboolean             expand;
   FlatpakEllipsizeMode ellipsize;
+  gboolean             skip_unique;
+  gboolean             skip;
 } TableColumn;
 
 static void
@@ -83,7 +85,7 @@ struct FlatpakTablePrinter
 {
   GPtrArray *columns;
   GPtrArray *rows;
-  char *key;
+  char      *key;
   GPtrArray *current;
   int        n_columns;
 };
@@ -110,6 +112,16 @@ flatpak_table_printer_free (FlatpakTablePrinter *printer)
   g_free (printer);
 }
 
+static TableColumn *
+peek_table_column (FlatpakTablePrinter *printer,
+                   int                  column)
+{
+  if (column < printer->columns->len)
+    return g_ptr_array_index (printer->columns, column);
+
+  return NULL;
+}
+
 static TableColumn *
 get_table_column (FlatpakTablePrinter *printer,
                   int                  column)
@@ -140,14 +152,18 @@ flatpak_table_printer_set_column_title (FlatpakTablePrinter *printer,
 
 void
 flatpak_table_printer_set_columns (FlatpakTablePrinter *printer,
-                                   Column              *columns)
+                                   Column              *columns,
+                                   gboolean             defaults)
 {
   int i;
+
   for (i = 0; columns[i].name; i++)
     {
       flatpak_table_printer_set_column_title (printer, i, _(columns[i].title));
       flatpak_table_printer_set_column_expand (printer, i, columns[i].expand);
       flatpak_table_printer_set_column_ellipsize (printer, i, columns[i].ellipsize);
+      if (defaults && columns[i].skip_unique_if_default)
+        flatpak_table_printer_set_column_skip_unique (printer, i, TRUE);
     }
 }
 
@@ -262,12 +278,12 @@ flatpak_table_printer_set_key (FlatpakTablePrinter *printer, const char *key)
 }
 
 static gint
-cmp_row (gconstpointer  _row_a,
-         gconstpointer  _row_b,
-         gpointer       user_data)
+cmp_row (gconstpointer _row_a,
+         gconstpointer _row_b,
+         gpointer      user_data)
 {
-  const Row *row_a = *(const Row **)_row_a;
-  const Row *row_b = *(const Row **)_row_b;
+  const Row *row_a = *(const Row **) _row_a;
+  const Row *row_b = *(const Row **) _row_b;
   GCompareFunc cmp = user_data;
 
   if (row_a == row_b || (row_a->key == NULL && row_b->key == NULL))
@@ -310,11 +326,11 @@ static int
 print_row (GString *row_s, gboolean bold, int *skip, int columns)
 {
   int rows;
-  char *p, *end;
+  const char *p, *end;
   int n_chars;
 
   g_strchomp (row_s->str);
-  n_chars = g_utf8_strlen (row_s->str, -1);
+  n_chars = cell_width (row_s->str);
   if (n_chars > 0)
     rows = (n_chars + columns - 1) / columns;
   else
@@ -325,13 +341,13 @@ print_row (GString *row_s, gboolean bold, int *skip, int columns)
   while (*skip > 0 && p <= end)
     {
       (*skip)--;
-      p = g_utf8_offset_to_pointer  (p, columns);
+      p = cell_advance (p, columns);
     }
 
   if (p < end || p == row_s->str)
     {
       if (bold)
-        g_print (FLATPAK_ANSI_BOLD_ON"%s"FLATPAK_ANSI_BOLD_OFF, p);
+        g_print (FLATPAK_ANSI_BOLD_ON "%s" FLATPAK_ANSI_BOLD_OFF, p);
       else
         g_print ("%s", p);
     }
@@ -347,6 +363,32 @@ string_add_spaces (GString *str, int count)
     g_string_append_c (str, ' ');
 }
 
+static gboolean
+colum_is_unique (FlatpakTablePrinter *printer, int col)
+{
+  char *first_row = NULL;
+  int i;
+
+  for (i = 0; i < printer->rows->len; i++)
+    {
+      Row *row = g_ptr_array_index (printer->rows, i);
+      if (col >= row->cells->len)
+        continue;
+
+      Cell *cell = g_ptr_array_index (row->cells, col);
+
+      if (i == 0)
+        first_row = cell->text;
+      else
+        {
+          if (g_strcmp0 (first_row, cell->text) != 0)
+            return FALSE;
+        }
+    }
+
+  return TRUE;
+}
+
 /*
  * This variant of flatpak_table_printer_print() takes a window width
  * and returns the number of rows that are generated by printing the
@@ -361,10 +403,10 @@ string_add_spaces (GString *str, int count)
  */
 void
 flatpak_table_printer_print_full (FlatpakTablePrinter *printer,
-                                  int skip,
-                                  int columns,
-                                  int *table_height,
-                                  int *table_width)
+                                  int                  skip,
+                                  int                  columns,
+                                  int                 *table_height,
+                                  int                 *table_width)
 {
   g_autofree int *widths = NULL;
   g_autofree int *lwidths = NULL;
@@ -388,14 +430,25 @@ flatpak_table_printer_print_full (FlatpakTablePrinter *printer,
   rwidths = g_new0 (int, printer->n_columns);
   shrinks = g_new0 (int, printer->n_columns);
 
+  for (i = 0; i < printer->columns->len && i < printer->n_columns; i++)
+    {
+      TableColumn *col = g_ptr_array_index (printer->columns, i);
+
+      if (col->skip_unique && colum_is_unique (printer, i))
+        col->skip = TRUE;
+    }
+
   has_title = FALSE;
   for (i = 0; i < printer->columns->len && i < printer->n_columns; i++)
     {
       TableColumn *col = g_ptr_array_index (printer->columns, i);
 
+      if (col->skip)
+        continue;
+
       if (col->title)
         {
-          widths[i] = MAX (widths[i], g_utf8_strlen (col->title, -1));
+          widths[i] = MAX (widths[i], cell_width (col->title));
           has_title = TRUE;
         }
     }
@@ -407,12 +460,16 @@ flatpak_table_printer_print_full (FlatpakTablePrinter *printer,
       for (j = 0; j < row->cells->len; j++)
         {
           Cell *cell = g_ptr_array_index (row->cells, j);
+          TableColumn *col = peek_table_column (printer, j);
           int width;
 
+          if (col && col->skip)
+            continue;
+
           if (cell->span)
             width = 0;
           else
-            width = g_utf8_strlen (cell->text, -1);
+            width = cell_width (cell->text);
           widths[j] = MAX (widths[j], width);
           if (cell->align >= 0)
             {
@@ -431,6 +488,8 @@ flatpak_table_printer_print_full (FlatpakTablePrinter *printer,
   for (i = 0; i < printer->columns->len; i++)
     {
       TableColumn *col = g_ptr_array_index (printer->columns, i);
+      if (col && col->skip)
+        continue;
       if (col && col->expand)
         expand_columns++;
       if (col && col->ellipsize)
@@ -465,11 +524,14 @@ flatpak_table_printer_print_full (FlatpakTablePrinter *printer,
               TableColumn *col = g_ptr_array_index (printer->columns, i);
               gboolean ellipsize = col ? col->ellipsize : FALSE;
 
+              if (col && col->skip)
+                continue;
+
               if (!ellipsize)
                 continue;
 
               if (col && col->title)
-                shrinkable += MAX (0, widths[i] - g_utf8_strlen (col->title, -1));
+                shrinkable += MAX (0, widths[i] - cell_width (col->title));
               else
                 shrinkable += MAX (0, widths[i] - 5);
             }
@@ -479,14 +541,17 @@ flatpak_table_printer_print_full (FlatpakTablePrinter *printer,
               TableColumn *col = g_ptr_array_index (printer->columns, i);
               gboolean ellipsize = col ? col->ellipsize : FALSE;
 
+              if (col && col->skip)
+                continue;
+
               if (ellipsize)
                 {
                   int sh;
                   if (col && col->title)
-                    sh = MAX (0, widths[i] - g_utf8_strlen (col->title, -1));
+                    sh = MAX (0, widths[i] - cell_width (col->title));
                   else
                     sh = MAX (0, widths[i] - 5);
-                  shrinks[i] = MIN (shortfall * (sh / (double)shrinkable), widths[i]);
+                  shrinks[i] = MIN (shortfall * (sh / (double) shrinkable), widths[i]);
                   leftover -= shrinks[i];
                 }
             }
@@ -499,10 +564,14 @@ flatpak_table_printer_print_full (FlatpakTablePrinter *printer,
                 {
                   TableColumn *col = g_ptr_array_index (printer->columns, i);
                   gboolean ellipsize = col ? col->ellipsize : FALSE;
+
+                  if (col && col->skip)
+                    continue;
+
                   if (ellipsize && shrinks[i] < widths[i])
                     {
-                       shrinks[i]++;
-                       leftover--;
+                      shrinks[i]++;
+                      leftover--;
                     }
                   if (leftover == 0)
                     break;
@@ -526,6 +595,9 @@ flatpak_table_printer_print_full (FlatpakTablePrinter *printer,
           int len = widths[i];
           g_autofree char *freeme = NULL;
 
+          if (col && col->skip)
+            continue;
+
           if (expand_by > 0 && expand)
             {
               len += expand_by;
@@ -545,7 +617,7 @@ flatpak_table_printer_print_full (FlatpakTablePrinter *printer,
           if (i > 0)
             g_string_append_c (row_s, ' ');
           g_string_append (row_s, title);
-          string_add_spaces (row_s, len - g_utf8_strlen (title, -1));
+          string_add_spaces (row_s, len - cell_width (title));
         }
       rows += print_row (row_s, TRUE, &skip, columns);
     }
@@ -560,7 +632,7 @@ flatpak_table_printer_print_full (FlatpakTablePrinter *printer,
 
       for (j = 0; j < row->cells->len; j++)
         {
-          TableColumn *col = j < printer->columns->len ? g_ptr_array_index (printer->columns, j) : NULL;
+          TableColumn *col = peek_table_column (printer, j);
           gboolean expand = col ? col->expand : FALSE;
           gboolean ellipsize = col ? col->ellipsize : FALSE;
           Cell *cell = g_ptr_array_index (row->cells, j);
@@ -568,6 +640,9 @@ flatpak_table_printer_print_full (FlatpakTablePrinter *printer,
           int len = widths[j];
           g_autofree char *freeme = NULL;
 
+          if (col && col->skip)
+            continue;
+
           if (expand_by > 0 && expand)
             {
               len += expand_by;
@@ -593,13 +668,13 @@ flatpak_table_printer_print_full (FlatpakTablePrinter *printer,
               else if (cell->align < 0)
                 {
                   g_string_append (row_s, text);
-                  string_add_spaces (row_s, len - g_utf8_strlen (text, -1));
+                  string_add_spaces (row_s, len - cell_width (text));
                 }
               else
                 {
                   string_add_spaces (row_s, lwidths[j] - cell->align);
                   g_string_append (row_s, text);
-                  string_add_spaces (row_s, widths[j] - (lwidths[j] - cell->align)  - g_utf8_strlen (text, -1));
+                  string_add_spaces (row_s, widths[j] - (lwidths[j] - cell->align)  - cell_width (text));
                 }
             }
           else
@@ -629,19 +704,19 @@ flatpak_table_printer_get_current_row (FlatpakTablePrinter *printer)
 
 static void
 set_cell (FlatpakTablePrinter *printer,
-          int r,
-          int c,
-          const char *text,
-          int align)
+          int                  r,
+          int                  c,
+          const char          *text,
+          int                  align)
 {
   Row *row;
   Cell *cell;
 
-  row = (Row *)g_ptr_array_index (printer->rows, r);
+  row = (Row *) g_ptr_array_index (printer->rows, r);
 
   g_assert (row);
 
-  cell = (Cell *)g_ptr_array_index (row->cells, c);
+  cell = (Cell *) g_ptr_array_index (row->cells, c);
   g_assert (cell);
 
   g_free (cell->text);
@@ -651,18 +726,18 @@ set_cell (FlatpakTablePrinter *printer,
 
 void
 flatpak_table_printer_set_cell (FlatpakTablePrinter *printer,
-                                int r,
-                                int c,
-                                const char *text)
+                                int                  r,
+                                int                  c,
+                                const char          *text)
 {
   set_cell (printer, r, c, text, -1);
 }
 
 void
 flatpak_table_printer_set_decimal_cell (FlatpakTablePrinter *printer,
-                                        int r,
-                                        int c,
-                                        const char *text)
+                                        int                  r,
+                                        int                  c,
+                                        const char          *text)
 {
   int align = -1;
   const char *decimal = find_decimal_point (text);
@@ -675,8 +750,8 @@ flatpak_table_printer_set_decimal_cell (FlatpakTablePrinter *printer,
 
 void
 flatpak_table_printer_set_column_expand (FlatpakTablePrinter *printer,
-                                         int column,
-                                         gboolean expand)
+                                         int                  column,
+                                         gboolean             expand)
 {
   TableColumn *col = get_table_column (printer, column);
 
@@ -685,10 +760,20 @@ flatpak_table_printer_set_column_expand (FlatpakTablePrinter *printer,
 
 void
 flatpak_table_printer_set_column_ellipsize (FlatpakTablePrinter *printer,
-                                            int column,
+                                            int                  column,
                                             FlatpakEllipsizeMode mode)
 {
   TableColumn *col = get_table_column (printer, column);
 
   col->ellipsize = mode;
 }
+
+void
+flatpak_table_printer_set_column_skip_unique (FlatpakTablePrinter *printer,
+                                              int                  column,
+                                              gboolean             skip_unique)
+{
+  TableColumn *col = get_table_column (printer, column);
+
+  col->skip_unique = skip_unique;
+}
