@@ -95,9 +95,9 @@ struct _FlatpakTransactionClass
 {
   GObjectClass parent_class;
 
-  void         (*new_operation)        (FlatpakTransaction          *transaction,
-                                        FlatpakTransactionOperation *operation,
-                                        FlatpakTransactionProgress  *progress);
+  void (*new_operation)        (FlatpakTransaction          *transaction,
+                                FlatpakTransactionOperation *operation,
+                                FlatpakTransactionProgress  *progress);
   void (*operation_done)       (FlatpakTransaction          *transaction,
                                 FlatpakTransactionOperation *operation,
                                 const char                  *commit,
@@ -121,7 +121,18 @@ struct _FlatpakTransactionClass
                               const char                    *from_id,
                               const char                    *remote_name,
                               const char                    *url);
-  gpointer padding[10];
+
+  gboolean (*run)            (FlatpakTransaction *transaction,
+                              GCancellable       *cancellable,
+                              GError            **error);
+  gboolean (*end_of_lifed_with_rebase) (FlatpakTransaction *transaction,
+                                        const char         *remote,
+                                        const char         *ref,
+                                        const char         *reason,
+                                        const char         *rebased_to_ref,
+                                        const char        **previous_ids);
+
+  gpointer padding[8];
 };
 
 FLATPAK_EXTERN
@@ -138,6 +149,10 @@ FLATPAK_EXTERN
 gboolean    flatpak_transaction_progress_get_is_estimating (FlatpakTransactionProgress *self);
 FLATPAK_EXTERN
 int         flatpak_transaction_progress_get_progress (FlatpakTransactionProgress *self);
+FLATPAK_EXTERN
+guint64     flatpak_transaction_progress_get_bytes_transferred (FlatpakTransactionProgress *self);
+FLATPAK_EXTERN
+guint64     flatpak_transaction_progress_get_start_time (FlatpakTransactionProgress *self);
 
 
 FLATPAK_EXTERN
@@ -151,6 +166,10 @@ GFile *                         flatpak_transaction_operation_get_bundle_path (F
 FLATPAK_EXTERN
 const char *                    flatpak_transaction_operation_get_commit (FlatpakTransactionOperation *self);
 FLATPAK_EXTERN
+guint64                         flatpak_transaction_operation_get_download_size (FlatpakTransactionOperation *self);
+FLATPAK_EXTERN
+guint64                         flatpak_transaction_operation_get_installed_size (FlatpakTransactionOperation *self);
+FLATPAK_EXTERN
 GKeyFile *                      flatpak_transaction_operation_get_metadata (FlatpakTransactionOperation *self);
 FLATPAK_EXTERN
 GKeyFile *                      flatpak_transaction_operation_get_old_metadata (FlatpakTransactionOperation *self);
@@ -190,7 +209,7 @@ void                flatpak_transaction_add_dependency_source (FlatpakTransactio
 FLATPAK_EXTERN
 void                flatpak_transaction_add_default_dependency_sources (FlatpakTransaction *self);
 FLATPAK_EXTERN
-gboolean            flatpak_transaction_run (FlatpakTransaction *self,
+gboolean            flatpak_transaction_run (FlatpakTransaction *transaction,
                                              GCancellable       *cancellable,
                                              GError            **error);
 FLATPAK_EXTERN
@@ -207,6 +226,13 @@ gboolean            flatpak_transaction_add_install (FlatpakTransaction *self,
                                                      const char        **subpaths,
                                                      GError            **error);
 FLATPAK_EXTERN
+gboolean            flatpak_transaction_add_rebase (FlatpakTransaction *self,
+                                                    const char         *remote,
+                                                    const char         *ref,
+                                                    const char        **subpaths,
+                                                    const char        **previous_ids,
+                                                    GError            **error);
+FLATPAK_EXTERN
 gboolean            flatpak_transaction_add_install_bundle (FlatpakTransaction *self,
                                                             GFile              *file,
                                                             GBytes             *gpg_data,
