@@ -1,4 +1,4 @@
-#!/bin/sh
+#!/bin/bash
 
 set -e
 
@@ -8,6 +8,8 @@ REPO=$1
 shift
 APP_ID=$1
 shift
+BRANCH=$1
+shift
 COLLECTION_ID=$1
 shift
 
@@ -23,8 +25,8 @@ ARCH=`flatpak --default-arch`
 cat > ${DIR}/metadata <<EOF
 [Application]
 name=$APP_ID
-runtime=org.test.Platform/$ARCH/master
-sdk=org.test.Platform/$ARCH/master
+runtime=org.test.Platform/$ARCH/$BRANCH
+sdk=org.test.Platform/$ARCH/$BRANCH
 EOF
 
 if [ x${REQUIRED_VERSION-} != x ]; then
@@ -34,7 +36,7 @@ EOF
 fi
 
 cat >> ${DIR}/metadata <<EOF
-[Extension org.test.Hello.Locale]
+[Extension $APP_ID.Locale]
 directory=share/runtime/locale
 autodelete=true
 locale-subset=true
@@ -60,6 +62,16 @@ Exec=hello.sh
 Icon=$APP_ID
 MimeType=x-test/Hello;
 EOF
+cat > ${DIR}/files/share/applications/org.test.Hello.Again.desktop <<EOF
+[Desktop Entry]
+Version=1.0
+Type=Application
+Name=Hello Again
+Exec=hello.sh --again
+Icon=$APP_ID
+MimeType=x-test/Hello;
+X-Flatpak-RenamedFrom=hello-again.desktop;
+EOF
 
 mkdir -p ${DIR}/files/share/icons/hicolor/64x64/apps
 cp $(dirname $0)/org.test.Hello.png ${DIR}/files/share/icons/hicolor/64x64/apps/${APP_ID}.png
@@ -104,7 +116,7 @@ ln -s -t ${DIR}/files/share/locale ../../share/runtime/locale/fr/share/fr
 
 flatpak build-finish --command=hello.sh ${DIR}
 mkdir -p repos
-flatpak build-export --disable-sandbox ${collection_args} ${GPGARGS-} ${EXPORT_ARGS-} ${REPO} ${DIR}
+flatpak build-export --disable-sandbox ${collection_args} ${GPGARGS-} ${EXPORT_ARGS-} ${REPO} ${DIR} ${BRANCH}
 rm -rf ${DIR}
 
 # build a locale extension
@@ -117,7 +129,7 @@ cat > ${DIR}/metadata <<EOF
 name=${APP_ID}.Locale
 
 [ExtensionOf]
-ref=app/$APP_ID/$ARCH/master
+ref=app/$APP_ID/$ARCH/$BRANCH
 EOF
 
 cat > de.po <<EOF
@@ -135,7 +147,7 @@ msgfmt --output-file ${DIR}/files/fr/share/fr/LC_MESSAGES/helloworld.mo fr.po
 
 flatpak build-finish ${DIR}
 mkdir -p repos
-flatpak build-export --runtime ${collection_args} ${GPGARGS-} ${EXPORT_ARGS-} ${REPO} ${DIR}
+flatpak build-export --runtime ${collection_args} ${GPGARGS-} ${EXPORT_ARGS-} ${REPO} ${DIR} ${BRANCH}
 rm -rf ${DIR}
 
 
