@@ -22,27 +22,30 @@ set -euo pipefail
 . $(dirname $0)/libtest.sh
 
 skip_without_bwrap
+skip_revokefs_without_fuse
 
-echo "1..13"
+echo "1..15"
 
-setup_repo
-install_repo
+# Use stable rather than master as the branch so we can test that the run
+# command automatically finds the branch correctly
+setup_repo "" "" stable
+install_repo "" stable
 
 # Verify that app is correctly installed
 
 assert_has_dir $FL_DIR/app/org.test.Hello
 assert_has_symlink $FL_DIR/app/org.test.Hello/current
-assert_symlink_has_content $FL_DIR/app/org.test.Hello/current ^$ARCH/master$
-assert_has_dir $FL_DIR/app/org.test.Hello/$ARCH/master
-assert_has_symlink $FL_DIR/app/org.test.Hello/$ARCH/master/active
-ID=`readlink $FL_DIR/app/org.test.Hello/$ARCH/master/active`
-assert_has_file $FL_DIR/app/org.test.Hello/$ARCH/master/active/deploy
-assert_has_file $FL_DIR/app/org.test.Hello/$ARCH/master/active/metadata
-assert_has_dir $FL_DIR/app/org.test.Hello/$ARCH/master/active/files
-assert_has_dir $FL_DIR/app/org.test.Hello/$ARCH/master/active/export
+assert_symlink_has_content $FL_DIR/app/org.test.Hello/current ^$ARCH/stable$
+assert_has_dir $FL_DIR/app/org.test.Hello/$ARCH/stable
+assert_has_symlink $FL_DIR/app/org.test.Hello/$ARCH/stable/active
+ID=`readlink $FL_DIR/app/org.test.Hello/$ARCH/stable/active`
+assert_has_file $FL_DIR/app/org.test.Hello/$ARCH/stable/active/deploy
+assert_has_file $FL_DIR/app/org.test.Hello/$ARCH/stable/active/metadata
+assert_has_dir $FL_DIR/app/org.test.Hello/$ARCH/stable/active/files
+assert_has_dir $FL_DIR/app/org.test.Hello/$ARCH/stable/active/export
 assert_has_file $FL_DIR/exports/share/applications/org.test.Hello.desktop
 # Ensure Exec key is rewritten
-assert_file_has_content $FL_DIR/exports/share/applications/org.test.Hello.desktop "^Exec=.*/flatpak run --branch=master --arch=$ARCH --command=hello.sh org.test.Hello$"
+assert_file_has_content $FL_DIR/exports/share/applications/org.test.Hello.desktop "^Exec=.*/flatpak run --branch=stable --arch=$ARCH --command=hello\.sh org\.test\.Hello$"
 assert_has_file $FL_DIR/exports/share/icons/hicolor/64x64/apps/org.test.Hello.png
 assert_not_has_file $FL_DIR/exports/share/icons/hicolor/64x64/apps/dont-export.png
 assert_has_file $FL_DIR/exports/share/icons/HighContrast/64x64/apps/org.test.Hello.png
@@ -69,18 +72,41 @@ assert_file_has_content hello_out '^Hello world, from a sandbox$'
 
 echo "ok hello"
 
-run_sh cat /run/user/`id -u`/flatpak-info > fpi
-assert_file_has_content fpi '^name=org.test.Hello$'
+run_sh org.test.Platform cat /.flatpak-info >runtime-fpi
+assert_file_has_content runtime-fpi "[Runtime]"
+assert_file_has_content runtime-fpi "^runtime=runtime/org\.test\.Platform/$ARCH/stable$"
+
+echo "ok run a runtime"
+
+if run org.test.Nonexistent 2> run-error-log; then
+    assert_not_reached "Unexpectedly able to run non-existent runtime"
+fi
+assert_file_has_content run-error-log "error: app/org\.test\.Nonexistent/$ARCH/master not installed"
+
+if ${FLATPAK} run --commit=abc runtime/org.test.Platform 2> run-error-log; then
+    assert_not_reached "Unexpectedly able to run non-existent commit"
+fi
+assert_file_has_content run-error-log "error: runtime/org\.test\.Platform/$ARCH/stable (commit abc) not installed"
+
+if run runtime/org.test.Nonexistent 2> run-error-log; then
+    assert_not_reached "Unexpectedly able to run non-existent runtime"
+fi
+assert_file_has_content run-error-log "error: runtime/org\.test\.Nonexistent/\*unspecified\*/\*unspecified\* not installed"
+
+echo "ok error handling for invalid refs"
+
+run_sh org.test.Hello cat /run/user/`id -u`/flatpak-info > fpi
+assert_file_has_content fpi '^name=org\.test\.Hello$'
 
 echo "ok flatpak-info"
 
-run_sh readlink /proc/self/ns/net > unshared_net_ns
-ARGS="--share=network" run_sh readlink /proc/self/ns/net > shared_net_ns
+run_sh org.test.Hello readlink /proc/self/ns/net > unshared_net_ns
+ARGS="--share=network" run_sh org.test.Hello readlink /proc/self/ns/net > shared_net_ns
 assert_not_streq `cat unshared_net_ns` `readlink /proc/self/ns/net`
 assert_streq `cat shared_net_ns` `readlink /proc/self/ns/net`
 
-run_sh readlink /proc/self/ns/ipc > unshared_ipc_ns
-ARGS="--share=ipc" run_sh readlink /proc/self/ns/ipc > shared_ipc_ns
+run_sh org.test.Hello readlink /proc/self/ns/ipc > unshared_ipc_ns
+ARGS="--share=ipc" run_sh org.test.Hello readlink /proc/self/ns/ipc > shared_ipc_ns
 assert_not_streq `cat unshared_ipc_ns` `readlink /proc/self/ns/ipc`
 assert_streq `cat shared_ipc_ns` `readlink /proc/self/ns/ipc`
 
@@ -95,13 +121,13 @@ assert_streq `cat shared_ipc_ns` `readlink /proc/self/ns/ipc`
 test_filesystem_binding () {
     local dir="$1"
 
-    if run_sh cat "$dir/package_version.txt" &> /dev/null; then
+    if run_sh org.test.Hello cat "$dir/package_version.txt" &> /dev/null; then
         assert_not_reached "Unexpectedly allowed to access file"
     fi
 
     case "$dir" in
         (/home/*|/opt/*|/var/tmp/*)
-            if ! ARGS="--filesystem=$dir" run_sh cat "$dir/package_version.txt" > /dev/null; then
+            if ! ARGS="--filesystem=$dir" run_sh org.test.Hello cat "$dir/package_version.txt" > /dev/null; then
                 assert_not_reached "Failed to share --filesystem=$dir"
             fi
             ;;
@@ -112,7 +138,7 @@ test_filesystem_binding () {
 
     case "$dir" in
         (/home/*|/opt/*)
-            if ! ARGS="--filesystem=host" run_sh cat "$dir/package_version.txt" > /dev/null; then
+            if ! ARGS="--filesystem=host" run_sh org.test.Hello cat "$dir/package_version.txt" > /dev/null; then
                 assert_not_reached "Failed to share $dir as part of host filesystem"
             fi
             ;;
@@ -142,7 +168,7 @@ echo "ok namespaces"
 test_overrides () {
     local dir="$1"
 
-    if run_sh cat "$dir/package_version.txt" &> /dev/null; then
+    if run_sh org.test.Hello cat "$dir/package_version.txt" &> /dev/null; then
         assert_not_reached "Unexpectedly allowed to access file"
     fi
 
@@ -150,7 +176,7 @@ test_overrides () {
 
     case "$dir" in
         (/home/*|/opt/*)
-            if ! run_sh cat "$dir/package_version.txt" > /dev/null; then
+            if ! run_sh org.test.Hello cat "$dir/package_version.txt" > /dev/null; then
                 assert_not_reached "Failed to share $dir as part of host filesystem"
             fi
             ;;
@@ -159,13 +185,13 @@ test_overrides () {
             ;;
     esac
 
-    if ARGS="--nofilesystem=host" run_sh cat "${dir}/package_version.txt" &> /dev/null; then
+    if ARGS="--nofilesystem=host" run_sh org.test.Hello cat "${dir}/package_version.txt" &> /dev/null; then
         assert_not_reached "Unexpectedly allowed to access --nofilesystem=host file"
     fi
 
     $FLATPAK override ${U} --nofilesystem=host org.test.Hello
 
-    if run_sh cat "${dir}/package_version.txt" &> /dev/null; then
+    if run_sh org.test.Hello cat "${dir}/package_version.txt" &> /dev/null; then
         assert_not_reached "Unexpectedly allowed to access file"
     fi
 }
@@ -185,14 +211,14 @@ OLD_COMMIT=`${FLATPAK} ${U} info --show-commit org.test.Hello`
 
 # TODO: For weird reasons this breaks in the system case. Needs debugging
 if [ x${USE_SYSTEMDIR-} != xyes ] ; then
-    ${FLATPAK} ${U} update -y -v org.test.Hello master
+    ${FLATPAK} ${U} update -y -v org.test.Hello stable
     ALSO_OLD_COMMIT=`${FLATPAK} ${U} info --show-commit org.test.Hello`
     assert_streq "$OLD_COMMIT" "$ALSO_OLD_COMMIT"
 fi
 
 echo "ok null update"
 
-make_updated_app
+make_updated_app "" "" stable
 
 ${FLATPAK} ${U} update -y org.test.Hello
 
@@ -205,7 +231,7 @@ assert_file_has_content hello_out '^Hello world, from a sandboxUPDATED$'
 
 echo "ok update"
 
-ostree --repo=repos/test reset app/org.test.Hello/$ARCH/master "$OLD_COMMIT"
+ostree --repo=repos/test reset app/org.test.Hello/$ARCH/stable "$OLD_COMMIT"
 update_repo
 
 if ${FLATPAK} ${U} update -y org.test.Hello; then
@@ -219,7 +245,7 @@ assert_streq "$NEW_COMMIT" "$NEW_NEW_COMMIT"
 echo "ok backwards update"
 
 DIR=`mktemp -d`
-${FLATPAK} build-init ${DIR} org.test.Split org.test.Platform org.test.Platform
+${FLATPAK} build-init ${DIR} org.test.Split org.test.Platform org.test.Platform stable
 
 mkdir -p ${DIR}/files/a
 echo "a" > ${DIR}/files/a/data
@@ -232,10 +258,10 @@ echo "d" > ${DIR}/files/d/data
 echo "nope" > ${DIR}/files/nope
 
 ${FLATPAK} build-finish --command=hello.sh ${DIR}
-${FLATPAK} build-export ${FL_GPGARGS} repos/test ${DIR}
+${FLATPAK} build-export ${FL_GPGARGS} repos/test ${DIR} stable
 update_repo
 
-${FLATPAK} ${U} install -y test-repo org.test.Split --subpath=/a --subpath=/b --subpath=/nosuchdir master
+${FLATPAK} ${U} install -y test-repo org.test.Split --subpath=/a --subpath=/b --subpath=/nosuchdir stable
 
 COMMIT=`${FLATPAK} ${U} info --show-commit org.test.Split`
 if [ x${USE_SYSTEMDIR-} != xyes ] ; then
@@ -243,12 +269,12 @@ if [ x${USE_SYSTEMDIR-} != xyes ] ; then
     assert_has_file $FL_DIR/repo/state/${COMMIT}.commitpartial
 fi
 
-assert_has_file $FL_DIR/app/org.test.Split/$ARCH/master/active/files/a/data
-assert_has_file $FL_DIR/app/org.test.Split/$ARCH/master/active/files/b/data
-assert_not_has_file $FL_DIR/app/org.test.Split/$ARCH/master/active/files/c
-assert_not_has_file $FL_DIR/app/org.test.Split/$ARCH/master/active/files/d
-assert_not_has_file $FL_DIR/app/org.test.Split/$ARCH/master/active/files/nope
-assert_not_has_file $FL_DIR/app/org.test.Split/$ARCH/master/active/files/nosuchdir
+assert_has_file $FL_DIR/app/org.test.Split/$ARCH/stable/active/files/a/data
+assert_has_file $FL_DIR/app/org.test.Split/$ARCH/stable/active/files/b/data
+assert_not_has_file $FL_DIR/app/org.test.Split/$ARCH/stable/active/files/c
+assert_not_has_file $FL_DIR/app/org.test.Split/$ARCH/stable/active/files/d
+assert_not_has_file $FL_DIR/app/org.test.Split/$ARCH/stable/active/files/nope
+assert_not_has_file $FL_DIR/app/org.test.Split/$ARCH/stable/active/files/nosuchdir
 
 echo "aa" > ${DIR}/files/a/data2
 rm  ${DIR}/files/a/data
@@ -258,7 +284,7 @@ mkdir -p ${DIR}/files/f
 echo "f" > ${DIR}/files/f/data
 rm -rf  ${DIR}/files/b
 
-${FLATPAK} build-export ${FL_GPGARGS} repos/test ${DIR}
+${FLATPAK} build-export ${FL_GPGARGS} repos/test ${DIR} stable
 update_repo
 
 ${FLATPAK} ${U} update -y --subpath=/a --subpath=/b --subpath=/e --subpath=/nosuchdir org.test.Split
@@ -269,16 +295,16 @@ if [ x${USE_SYSTEMDIR-} != xyes ] ; then
     assert_has_file $FL_DIR/repo/state/${COMMIT}.commitpartial
 fi
 
-assert_not_has_file $FL_DIR/app/org.test.Split/$ARCH/master/active/files/a/data
-assert_has_file $FL_DIR/app/org.test.Split/$ARCH/master/active/files/a/data2
-assert_not_has_file $FL_DIR/app/org.test.Split/$ARCH/master/active/files/b
-assert_not_has_file $FL_DIR/app/org.test.Split/$ARCH/master/active/files/c
-assert_not_has_file $FL_DIR/app/org.test.Split/$ARCH/master/active/files/d
-assert_has_file $FL_DIR/app/org.test.Split/$ARCH/master/active/files/e/data
-assert_not_has_file $FL_DIR/app/org.test.Split/$ARCH/master/active/files/f
-assert_not_has_file $FL_DIR/app/org.test.Split/$ARCH/master/active/files/nope
+assert_not_has_file $FL_DIR/app/org.test.Split/$ARCH/stable/active/files/a/data
+assert_has_file $FL_DIR/app/org.test.Split/$ARCH/stable/active/files/a/data2
+assert_not_has_file $FL_DIR/app/org.test.Split/$ARCH/stable/active/files/b
+assert_not_has_file $FL_DIR/app/org.test.Split/$ARCH/stable/active/files/c
+assert_not_has_file $FL_DIR/app/org.test.Split/$ARCH/stable/active/files/d
+assert_has_file $FL_DIR/app/org.test.Split/$ARCH/stable/active/files/e/data
+assert_not_has_file $FL_DIR/app/org.test.Split/$ARCH/stable/active/files/f
+assert_not_has_file $FL_DIR/app/org.test.Split/$ARCH/stable/active/files/nope
 
-${FLATPAK} build-export ${FL_GPGARGS} repos/test ${DIR}
+${FLATPAK} build-export ${FL_GPGARGS} repos/test ${DIR} stable
 update_repo
 
 # Test reusing the old subpath list
@@ -290,49 +316,49 @@ if [ x${USE_SYSTEMDIR-} != xyes ] ; then
     assert_has_file $FL_DIR/repo/state/${COMMIT}.commitpartial
 fi
 
-assert_not_has_file $FL_DIR/app/org.test.Split/$ARCH/master/active/files/a/data
-assert_has_file $FL_DIR/app/org.test.Split/$ARCH/master/active/files/a/data2
-assert_not_has_file $FL_DIR/app/org.test.Split/$ARCH/master/active/files/b
-assert_not_has_file $FL_DIR/app/org.test.Split/$ARCH/master/active/files/c
-assert_not_has_file $FL_DIR/app/org.test.Split/$ARCH/master/active/files/d
-assert_has_file $FL_DIR/app/org.test.Split/$ARCH/master/active/files/e/data
-assert_not_has_file $FL_DIR/app/org.test.Split/$ARCH/master/active/files/f
-assert_not_has_file $FL_DIR/app/org.test.Split/$ARCH/master/active/files/nope
+assert_not_has_file $FL_DIR/app/org.test.Split/$ARCH/stable/active/files/a/data
+assert_has_file $FL_DIR/app/org.test.Split/$ARCH/stable/active/files/a/data2
+assert_not_has_file $FL_DIR/app/org.test.Split/$ARCH/stable/active/files/b
+assert_not_has_file $FL_DIR/app/org.test.Split/$ARCH/stable/active/files/c
+assert_not_has_file $FL_DIR/app/org.test.Split/$ARCH/stable/active/files/d
+assert_has_file $FL_DIR/app/org.test.Split/$ARCH/stable/active/files/e/data
+assert_not_has_file $FL_DIR/app/org.test.Split/$ARCH/stable/active/files/f
+assert_not_has_file $FL_DIR/app/org.test.Split/$ARCH/stable/active/files/nope
 
 echo "ok subpaths"
 
 VERSION=`cat "$test_builddir/package_version.txt"`
 
 DIR=`mktemp -d`
-${FLATPAK} build-init ${DIR} org.test.CurrentVersion org.test.Platform org.test.Platform
+${FLATPAK} build-init ${DIR} org.test.CurrentVersion org.test.Platform org.test.Platform stable
 ${FLATPAK} build-finish --require-version=${VERSION} --command=hello.sh ${DIR}
-${FLATPAK} build-export ${FL_GPGARGS} repos/test ${DIR}
+${FLATPAK} build-export ${FL_GPGARGS} repos/test ${DIR} stable
 DIR=`mktemp -d`
-${FLATPAK} build-init ${DIR} org.test.OldVersion org.test.Platform org.test.Platform
+${FLATPAK} build-init ${DIR} org.test.OldVersion org.test.Platform org.test.Platform stable
 ${FLATPAK} build-finish --require-version=0.6.10 --command=hello.sh ${DIR}
-${FLATPAK} build-export ${FL_GPGARGS} repos/test ${DIR}
+${FLATPAK} build-export ${FL_GPGARGS} repos/test ${DIR} stable
 DIR=`mktemp -d`
-${FLATPAK} build-init ${DIR} org.test.NewVersion org.test.Platform org.test.Platform
+${FLATPAK} build-init ${DIR} org.test.NewVersion org.test.Platform org.test.Platform stable
 ${FLATPAK} build-finish --require-version=1${VERSION} --command=hello.sh ${DIR}
-${FLATPAK} build-export ${FL_GPGARGS} repos/test ${DIR}
+${FLATPAK} build-export ${FL_GPGARGS} repos/test ${DIR} stable
 
 update_repo
 
-${FLATPAK} ${U} install -y test-repo org.test.OldVersion master
-${FLATPAK} ${U} install -y test-repo org.test.CurrentVersion master
-(! ${FLATPAK} ${U} install -y test-repo org.test.NewVersion master)
+${FLATPAK} ${U} install -y test-repo org.test.OldVersion stable
+${FLATPAK} ${U} install -y test-repo org.test.CurrentVersion stable
+(! ${FLATPAK} ${U} install -y test-repo org.test.NewVersion stable)
 
 DIR=`mktemp -d`
-${FLATPAK} build-init ${DIR} org.test.OldVersion org.test.Platform org.test.Platform
+${FLATPAK} build-init ${DIR} org.test.OldVersion org.test.Platform org.test.Platform stable
 ${FLATPAK} build-finish --require-version=99.0.0 --command=hello.sh ${DIR}
-${FLATPAK} build-export ${FL_GPGARGS} repos/test ${DIR}
+${FLATPAK} build-export ${FL_GPGARGS} repos/test ${DIR} stable
 
 (! ${FLATPAK} ${U} update -y org.test.OldVersion)
 
 DIR=`mktemp -d`
-${FLATPAK} build-init ${DIR} org.test.OldVersion org.test.Platform org.test.Platform
+${FLATPAK} build-init ${DIR} org.test.OldVersion org.test.Platform org.test.Platform stable
 ${FLATPAK} build-finish --require-version=0.1.1 --command=hello.sh ${DIR}
-${FLATPAK} build-export ${FL_GPGARGS} repos/test ${DIR}
+${FLATPAK} build-export ${FL_GPGARGS} repos/test ${DIR} stable
 
 ${FLATPAK} ${U} update -y org.test.OldVersion
 
@@ -341,49 +367,52 @@ ${FLATPAK} ${U} update -y org.test.OldVersion
 # and should not block a successful install of OldVersion later
 
 DIR=`mktemp -d`
-${FLATPAK} build-init ${DIR} org.test.CurrentVersion org.test.Platform org.test.Platform
+${FLATPAK} build-init ${DIR} org.test.CurrentVersion org.test.Platform org.test.Platform stable
 touch ${DIR}/files/updated
 ${FLATPAK} build-finish --require-version=99.0.0 --command=hello.sh ${DIR}
-${FLATPAK} build-export ${FL_GPGARGS} repos/test ${DIR}
+${FLATPAK} build-export ${FL_GPGARGS} repos/test ${DIR} stable
 DIR=`mktemp -d`
-${FLATPAK} build-init ${DIR} org.test.OldVersion org.test.Platform org.test.Platform
+${FLATPAK} build-init ${DIR} org.test.OldVersion org.test.Platform org.test.Platform stable
 touch ${DIR}/files/updated
 ${FLATPAK} build-finish --require-version=${VERSION} --command=hello.sh ${DIR}
-${FLATPAK} build-export ${FL_GPGARGS} repos/test ${DIR}
+${FLATPAK} build-export ${FL_GPGARGS} repos/test ${DIR} stable
 
 if ${FLATPAK} ${U} update -y &> err_version.txt; then
     assert_not_reached "Should not have been able to update due to version"
 fi
 assert_file_has_content err_version.txt "needs a later flatpak version"
 
-assert_not_has_file $FL_DIR/app/org.test.CurrentVersion/$ARCH/master/active/files/updated
-assert_has_file $FL_DIR/app/org.test.OldVersion/$ARCH/master/active/files/updated
+assert_not_has_file $FL_DIR/app/org.test.CurrentVersion/$ARCH/stable/active/files/updated
+assert_has_file $FL_DIR/app/org.test.OldVersion/$ARCH/stable/active/files/updated
 
 echo "ok version checks"
 
 rm -rf app
-flatpak build-init app org.test.Writable org.test.Platform org.test.Platform
+flatpak build-init app org.test.Writable org.test.Platform org.test.Platform stable
 mkdir -p app/files/a-dir
 chmod a+rwx app/files/a-dir
 flatpak build-finish --command=hello.sh app
 # Note: not --canonical-permissions
-ostree --repo=repos/test commit --owner-uid=0 --owner-gid=0  --no-xattrs  ${FL_GPGARGS} --branch=app/org.test.Writable/$ARCH/master app
+ostree --repo=repos/test commit --owner-uid=0 --owner-gid=0  --no-xattrs  ${FL_GPGARGS} --branch=app/org.test.Writable/$ARCH/stable app
 update_repo
 
-${FLATPAK} ${U} install -y test-repo org.test.Writable
-
-assert_file_has_mode $FL_DIR/app/org.test.Writable/$ARCH/master/active/files/a-dir 775
+# In the system-helper case this fails to install due to the permission canonicalization happening in the
+# child-repo making objects get the wrong checksum, whereas in the user case we successfully import it, but
+# it will have canonicalized permissions.
+if ${FLATPAK} ${U} install -y test-repo org.test.Writable; then
+    assert_file_has_mode $FL_DIR/app/org.test.Writable/$ARCH/stable/active/files/a-dir 775
+fi
 
 echo "ok no world writable dir"
 
 rm -rf app
-flatpak build-init app org.test.Setuid org.test.Platform org.test.Platform
+flatpak build-init app org.test.Setuid org.test.Platform org.test.Platform stable
 mkdir -p app/files/
 touch app/files/exe
 chmod u+s app/files/exe
 flatpak build-finish --command=hello.sh app
 # Note: not --canonical-permissions
-ostree --repo=repos/test commit --owner-uid=0 --owner-gid=0 --no-xattrs  ${FL_GPGARGS} --branch=app/org.test.Setuid/$ARCH/master app
+ostree --repo=repos/test commit --owner-uid=0 --owner-gid=0 --no-xattrs  ${FL_GPGARGS} --branch=app/org.test.Setuid/$ARCH/stable app
 update_repo
 
 if ${FLATPAK} ${U} install -y test-repo org.test.Setuid &> err2.txt; then
@@ -394,16 +423,16 @@ assert_file_has_content err2.txt [Ii]nvalid
 echo "ok no setuid"
 
 rm -rf app
-flatpak build-init app org.test.App org.test.Platform org.test.Platform
+flatpak build-init app org.test.App org.test.Platform org.test.Platform stable
 mkdir -p app/files/
 touch app/files/exe
 flatpak build-finish --command=hello.sh --sdk=org.test.Sdk app
-${FLATPAK} build-export ${FL_GPGARGS} repos/test app
+${FLATPAK} build-export ${FL_GPGARGS} repos/test app stable
 update_repo
 
 ${FLATPAK} ${U} install -y test-repo org.test.App
 ${FLATPAK} ${U} info -m org.test.App > out
 
-assert_file_has_content out "^sdk=org.test.Sdk/$(flatpak --default-arch)/master$"
+assert_file_has_content out "^sdk=org\.test\.Sdk/$(flatpak --default-arch)/stable$"
 
 echo "ok --sdk option"
